package fms

import (
	"encoding/json"
	"errors"
	"fmt"
	"github.com/shopspring/decimal"
	"hongze/fms_api/models/fms"
	"hongze/fms_api/services/alarm_msg"
	"hongze/fms_api/services/resource"
	"hongze/fms_api/utils"
	"os"
	"regexp"
	"strings"
	"time"
)

// CheckContractRegisterAmount 校验合同登记金额-判断是否需要更新合同登记状态
func CheckContractRegisterAmount(registerId int) {
	var err error
	defer func() {
		if err != nil {
			alarm_msg.SendAlarmMsg("校验合同登记金额失败, ErrMsg: "+err.Error(), 3)
		}
	}()

	// 获取合同登记信息
	cr := new(fms.ContractRegister)
	item, e := cr.Fetch(registerId)
	if e != nil {
		err = errors.New("获取合同登记信息失败, Err: " + e.Error())
		return
	}
	amount := decimal.NewFromFloat(item.ContractAmount).Round(2)

	// 获取开票/到款信息
	invoiceCond := `contract_register_id = ?`
	invoicePars := make([]interface{}, 0)
	invoicePars = append(invoicePars, registerId)
	ci := new(fms.ContractInvoice)
	invoiceList, e := ci.List(invoiceCond, invoicePars)
	if e != nil {
		err = errors.New("获取开票到款信息失败, Err: " + e.Error())
		return
	}
	invoiceAmount := decimal.NewFromFloat(0).Round(2)
	paymentAmount := decimal.NewFromFloat(0).Round(2)
	for i := range invoiceList {
		a := decimal.NewFromFloat(invoiceList[i].Amount).Round(2)
		if invoiceList[i].InvoiceType == fms.ContractInvoiceTypeMake {
			invoiceAmount = invoiceAmount.Add(a)
		}
		if invoiceList[i].InvoiceType == fms.ContractInvoiceTypePay {
			paymentAmount = paymentAmount.Add(a)
		}
	}

	// 核对金额是否相等, 并更新状态
	status := fms.ContractRegisterStatusIng
	if amount.Equal(invoiceAmount) && invoiceAmount.Equal(paymentAmount) && item.ContractStatus == fms.ContractStatusCheckedBack {
		status = fms.ContractRegisterStatusComplete
	}
	updateCols := []string{"InvoicedAmount", "PaymentAmount", "RegisterStatus", "ModifyTime"}
	ia, _ := invoiceAmount.Round(2).Float64()
	pa, _ := paymentAmount.Round(2).Float64()
	item.InvoicedAmount = ia
	item.PaymentAmount = pa
	item.RegisterStatus = status
	item.ModifyTime = time.Now().Local()
	if e = item.Update(updateCols); e != nil {
		err = errors.New("更新合同登记金额及状态失败, Err: " + e.Error())
		return
	}
	return
}

// HandleContractServiceAndDetail 根据请求数据获取套餐服务详情信息
func HandleContractServiceAndDetail(productId int, serviceList []fms.ContractServiceAddReq, base642Image bool) (serviceDetailList []*fms.ContractServiceAndDetail, err error) {
	serviceDetailList = make([]*fms.ContractServiceAndDetail, 0)
	if len(serviceList) < 0 {
		err = errors.New("请至少选择一种套餐")
		return
	}
	for i := 0; i < len(serviceList); i++ {
		item := serviceList[i]
		detail := item.Detail

		detailList := make([]*fms.ContractServiceDetail, 0)
		if len(detail) > 0 {
			for j := 0; j < len(detail); j++ {
				detailItem := detail[j]
				cellMap := make(map[string]string)
				for k := 0; k < len(detailItem); k++ {
					key := fmt.Sprint("cell_", k+1)
					v, e := json.Marshal(detailItem[k])
					if e != nil {
						err = errors.New(fmt.Sprint("json转换失败：", e))
						return
					}
					cellMap[key] = string(v)
				}
				contractServiceDetail := &fms.ContractServiceDetail{
					Col1: func(cellMap map[string]string) string {
						v, ok := cellMap["cell_1"]
						if ok {
							return v
						} else {
							return ""
						}
					}(cellMap),
					Col2: func(cellMap map[string]string) string {
						v, ok := cellMap["cell_2"]
						if ok {
							return v
						} else {
							return ""
						}
					}(cellMap),
					Col3: func(cellMap map[string]string) string {
						v, ok := cellMap["cell_3"]
						if ok {
							return v
						} else {
							return ""
						}
					}(cellMap),
					Col4: func(cellMap map[string]string) string {
						v, ok := cellMap["cell_4"]
						if ok {
							return v
						} else {
							return ""
						}
					}(cellMap),
					Col5: func(cellMap map[string]string) string {
						v, ok := cellMap["cell_5"]
						if ok {
							return v
						} else {
							return ""
						}
					}(cellMap),
					Col6: func(cellMap map[string]string) string {
						v, ok := cellMap["cell_6"]
						if ok {
							return v
						} else {
							return ""
						}
					}(cellMap),
					Col7: func(cellMap map[string]string) string {
						v, ok := cellMap["cell_7"]
						if ok {
							return v
						} else {
							return ""
						}
					}(cellMap),
					ServiceTemplateId: item.ServiceTemplateId,
					CreateTime:        time.Now(),
				}
				detailList = append(detailList, contractServiceDetail)
			}
		}
		hasDetail := "否"
		if len(detailList) > 0 {
			hasDetail = "是"
		}
		// 报价单图片地址
		newValue := item.Value
		if base642Image {
			b, _ := regexp.MatchString(`^data:\s*image\/(\w+);base64,`, newValue)
			if b {
				imageUrl, e := UploadImageBase64(newValue)
				if e != nil {
					err = errors.New(fmt.Sprint("base64图片上传失败：", e))
					return
				}
				newValue = imageUrl
			}
		}

		// 合同模板
		serviceTemp, e := fms.GetContractServiceTemplateById(item.ServiceTemplateId)
		if e != nil {
			err = errors.New("找不到该模板")
			return
		}
		serviceDetail := &fms.ContractServiceAndDetail{
			ServiceTemplateId:  item.ServiceTemplateId,
			Title:              item.Title,
			Value:              newValue,
			ProductId:          productId,
			HasDetail:          hasDetail,
			TableValue:         serviceTemp.TableValue,
			Detail:             detailList,
			ChartPermissionId:  serviceTemp.ChartPermissionId,
			ChartPermissionIds: item.ChartPermissionIds,
		}
		serviceDetailList = append(serviceDetailList, serviceDetail)
	}
	return
}

// GetContractServiceAndDetail 根据合同登记ID获取服务详情
func GetContractServiceAndDetail(contractRegisterId int) (serviceList []*fms.ContractServiceAndDetail, err error) {
	list, e := fms.GetContractServiceAndDetailList(contractRegisterId)
	if e != nil {
		err = errors.New(fmt.Sprint("查找合同服务异常", e))
		return
	}
	serviceList = list
	newLen := len(serviceList)
	for i := 0; newLen > i; i++ {
		if serviceList[i].HasDetail == "是" {
			detail, e := fms.GetContractServiceDetailByServiceId(serviceList[i].ContractServiceId)
			if e != nil {
				err = errors.New(fmt.Sprint("查找合同服务详情异常", e))
				return
			}
			serviceList[i].Detail = detail
		}
	}
	return
}

// UploadImageBase64 将base64的图片上传至oss
func UploadImageBase64(base64Str string) (resourceUrl string, err error) {
	ext := ".png"
	uploadDir := "./static"
	randStr := utils.GetRandStringNoSpecialChar(28)
	fileName := randStr + ext
	fpath := uploadDir + "/" + fileName

	b, _ := regexp.MatchString(`^data:\s*image\/(\w+);base64,`, base64Str)
	if !b {
		err = errors.New("图片格式不正确")
		return
	}
	re, _ := regexp.Compile(`^data:\s*image\/(\w+);base64,`)
	base64Str = re.ReplaceAllString(base64Str, "")
	base64Str = strings.Replace(base64Str, " ", "", -1)

	base64Str = strings.Replace(base64Str, " ", "", -1)
	err = utils.SaveBase64ToFile(base64Str, fpath)
	if err != nil {
		return
	}
	defer os.Remove(fpath)

	savePath := utils.OssUploadImgDir + time.Now().Format("200601/20060102/")
	savePath += fileName
	// 上传到阿里云
	err = resource.UploadVideoAliyun(fileName, fpath, savePath)
	if err != nil {
		return
	}
	resourceUrl = utils.OssHost + savePath
	return
}
